module mpi_interface

#if (MPI_MPI_INTEGER_TYPE == 4)
use mpi
#endif


implicit none
!private ! do not use private here, so that mpif.h or mpi module will be public

#if (MPI_MPI_INTEGER_TYPE == 8)
  include 'mpif.h'
#endif

  integer, parameter :: I8B = selected_int_kind(12)  ! 8-byte integer
  integer, parameter :: I4B = selected_int_kind(9)   ! 4-byte integer

   ! this is the system integer type
   integer, parameter, public :: sys_int_kind = kind(0)

   ! this is the integer type for mpi control values 
#if (MPI_MPI_INTEGER_TYPE == 4)
   integer, parameter, public :: mpi_int_kind = I4B
#elif (MPI_MPI_INTEGER_TYPE == 8)
   integer, parameter, public :: mpi_int_kind = I8B   
#endif    

end module mpi_interface

!*************************************************************************************************
program test
use mpi_interface
implicit none

   integer(mpi_int_kind) :: ierr, nproc, rank
   logical :: flag

#ifdef FOUR_BYTE_ATTRIB
   integer(I4B) :: attrib_val ! Accordint to intel mpi mautal, MPI_TAB_UB attribute always a 4-byte integer ??
#else
   integer(MPI_ADDRESS_KIND) :: attrib_val
#endif   

   call MPI_INIT(ierr)
   call MPI_COMM_SIZE(MPI_COMM_WORLD,nproc,ierr)
   call MPI_COMM_RANK(MPI_COMM_WORLD,rank,ierr)
   write(*,*) ' NPROC: ', nproc
   write(*,*) ' RANK: ', nproc
   write(*,*)

   write(*,'(5A25)') 'I8B','I4V','syst_int_kind','MPI_ADDRESS_KIND','kind(attrib_val)'
   write(*,'(5I25)') I8B,I4B,sys_int_kind,MPI_ADDRESS_KIND,kind(attrib_val)

   call MPI_COMM_GET_ATTR(MPI_COMM_WORLD,MPI_TAG_UB,attrib_val,flag,ierr)
   if ( .not. flag ) write(*,*) 'No MPI_TAB_UB attribute found'
   if ( ierr .ne. 0 ) then
      write(*,'(A,I,A)') 'MPI_COMM_GET_ATTR returned error status ', ierr, ' in mpi_max_tag_value'
   end if
   write(*,*)
   write(*,'(A,I20)') ' ATTRIB_VAL: ', attrib_val

   call MPI_FINALIZE(ierr)

end program test
!*************************************************************************************************


